#!/usr/bin/python
# -*- coding: UTF-8 -*-

from tkinter import *
from tkinter import messagebox

# ---  CONFIG ------------------------------------------------------------------
#
# ------------------------------------------------------------------------------

class Config(object):
    def __init__(self, master):

        self.master = master
        self.rx = None                    # um Daten an den Receiver zu senden
 #       self.datei_is_open = False
        self.crc           = IntVar()
        self.msg           = IntVar()
        self.itv           = IntVar()
        self.crc.set(0)
        self.itv.set(-1)    # damit zunächst kein Checkbutton aktiviert ist
        self.msg.set(-1)


##    def set_connection(self, rx):
##        self.rx = rx


    def config_window(self, rx):
        """ Hier kann der GSS-Empänger Navilock NL 303P konfiguriert werden
        """
        self.rx = rx
        self.w = Toplevel()
        self.w.title("GPS-Receiver-Konfiguration")
        self.w.geometry('+200+150')

        self.frm_msg = LabelFrame(self.w, width = 20, text = "Message-Typ")
        self.frm_msg.grid(column = 0, row = 0, rowspan = 7, padx = 10,
                                        pady = 10, sticky = N+S+W+E)

        self.frm_int = LabelFrame(self.w, width = 20, text = "Sende-Intervall")
        self.frm_int.grid(column = 1, row = 0, padx = 10, rowspan = 7,
                                        pady = 10, sticky = N+S+W+E)

        self.frm_crc = LabelFrame(self.w, width = 20, text = "Prüfsumme")
        self.frm_crc.grid(column = 2, row = 0, padx = 10, rowspan = 7,
                                        pady = 10, sticky = N+S+W+E)

        Button(self.w, text="übertragen", command=self._gps_config,
                        width = 15).grid(column=4, row=5, padx = 10, sticky=W)
        Button(self.w, text="Abbrechen", command= self._wdw_ende ,
                        width = 15).grid(column=4, row=6, padx = 10, sticky=W)


        items = (("GPGGA", 0),
                 ("GPGLL", 1),
                 ("GPGSA", 2),
                 ("GPGSV", 3),
                 ("GPRMC", 4),
                 ("GPVTG", 5),)

        row = 0
        for i in items:
            Radiobutton(self.frm_msg, text = i[0], variable= self.msg, anchor = W,
                    value = i[1], width = 10).grid(
                    column = 0, row = row, padx = 5, sticky = E)
            row += 1

        items = (("-AUS-", 0),
                 ("1 Sek", 1),
                 ("5 Sek", 5),
                 ("10Sek", 10),
                 ("30Sek", 30),
                 ("60Sek", 60),)

        row = 0
        for i in items:
            Radiobutton(self.frm_int, text =i[0], variable= self.itv, anchor = W,
                value = i[1], width = 10).grid(column = 0, row = row, padx = 5, sticky = E)
            row += 1

        items = (("ohneCRC", 0),
                ("mit CRC", 1),)

        row = 0
        for i in items:
            Radiobutton(self.frm_crc, text = i[0], variable= self.crc, anchor = W,
                value = i[1], width = 10).grid(column = 0, row = row, padx = 5, sticky = E)
            row += 1

        self.w.focus_set()                # den Focus auf das Fenster setzen
        self.w.transient(self.master)     # das Fenster im Vordergrund vor "root" halten
        self.w.grab_set()                 # events gehen nur an diesen Dialog
#       self.master.wait_window(self.w)   # warten bis das Fenster geschlossen wird


    def _gps_config(self):
        """ Hier werden die im Config-Window gewählten Optionen an den Empfänger
            übertragen """
        a = self.msg.get()
        b = self.itv.get()
        c = self.crc.get()

        if not ((a in (0,1,2,3,4,5)) and (b in (0, 1, 5, 10, 30, 60)) and (c in (0, 1))):
            messagebox.showerror("GPS", "Message-Typ, Sende-Intervall und CRC\nauswählen !")
            return
        s = self._print_cmd_103(a, b, c)
        s = s + "\r\n"

        if messagebox.askyesno("GPS", "Den Konfigurations-String:\n\n'"+  repr(s)
                                    + "'\n\n" + "an GPS-Receiver senden ?"):
            print(s)
            if self.rx is not None:
                x = self.rx.write(s.encode(encoding = 'ascii'))
                messagebox.showinfo("GPS", str(x) + " Bytes geschrieben !")
            else:
                messagebox.showerror("GPS", "Keine Serielle Schnittstelle geöffnet !")

        self.w.destroy()


# ------------------------------------------------------------------------------
    def _print_cmd_103(self, msg, rate=0, check = 1):

        """ Konfiguration für den NAVILOCK

            msg   : [GGA, GLL, GSA, GSV, RMC, VTG]
            mode  : fest auf SetRate eingestellt
            rate  : 0 = AUS, sonst seconds
            check : 0 = AUS - Prüfsumme senden
        """
        dic = {0: "00", 1: "01", 2:"02", 3:"03", 4: "04", 5:"05"}

        t1 = "$PSRF103,"
        t2 = dic.get(msg) + ",00,"   # Message + Mode = 0 = SetRate

        t3 = hex(rate)               # bis FF = 255
        t3 = t3[2:]
        t3 = t3.rjust(2,'0')         # rate
        t3 = t3.upper()

        if check == 0:               # checksumme
            t4 = ",00*"
        else:
            t4 = ",01*"

        t = t1 + t2 + t3 + t4       # alles zusammenfügen
        tt = t.encode()             # und in Ziffernfolge umwandeln
        check = 0                   # für die Prüfsummen
                                    # beginnend nach dem $, aber ohne den '*'
        for i in range(1, len(tt) - 1):
            check = check ^ (tt[i])

        t1 = hex(check)
        t1 = t1[2:]                 # 0x abschneiden
        t1 = t1.rjust(2,'0')        # auf zwei Stellen auffüllen
        t1 = t1.upper()
        return t + t1


    def _wdw_ende(self):
        """ Beendet das modale Fenster """
        self.w.destroy()

