#!/usr/bin/python
# -*- coding: UTF-8 -*-

# --- [ gps_main.py ] -------------------------------------------

from gps_config import *
from gps_window import *
from gps_msg    import *
from serport import open_serport
from file_open import file_open

from threading import Thread
import time
from tkinter import *
from tkinter import messagebox


# ------------------------------------------------------------------------------
class App(Tk):

    Titel_1 = "GPS-Receiver"
    Titel_2 = ""

    def __init__(self):
        Tk.__init__(self)

        self.title(App.Titel_1)
        self.resizable(width = False, height = False)
        self.bind_class("Tk", "<Destroy>", self.__shutdown)

        # die Tasten 'p' und 'r' zuweisen
        self.bind_all("<KeyPress-p>", self.__resetref)
        self.bind_all("<KeyPress-r>", self.__resetminmax)

        self.rx = None                  # keine Serielle Schnittstelle
        self.handle = None              # keine Datei geöffnet
        self.filename = ""

        self.addMenus()
        self.addWidgets()
        self.addConnections()

        self.mainloop()


    def addMenus(self):
        self.log_file  = IntVar()       # für MenuCheckbutton
        self.log_activ = IntVar()       # für MenuCheckbutton
        self.log_activ.set(1)           # das Loggen ist aktiv

        self.menuleiste = Menu(self)
#        print(type(self.menuleiste))
#        self.configure(menu = self.menuleiste)
        self['menu'] = self.menuleiste

        self.dateimenue = Menu(master = self.menuleiste)
        self.dateimenue.add_command(label = "Einstellungen",
                                        command = self.gps_config)
        self.dateimenue.add_separator()

        self.dateimenue.add_checkbutton(label = "GPS-Daten einlesen", variable = self.log_activ,
                                onvalue = 1, offvalue = 0, command = self.logpause)
        self.dateimenue.add_checkbutton(label = "GPS-Daten loggen", variable = self.log_file,
                                onvalue = 1, offvalue = 0, command = self.logfile)
        self.dateimenue.add_separator()
        self.dateimenue.add_command(label = "Beenden", command = self.ende)
        self.menuleiste.add_cascade(label = "Datei", menu = self.dateimenue)


    def addWidgets(self):
        self.rmc = RMC(self)
        self.gsa = GSA(self)
        self.gga = GGA(self)
        self.gsv = GSV(self)
        self.gsv = GSV(self)
        self.gll = GLL(self)

        self.counter = Frm_counter(self)
        self.minmax = Minmax(self)
        self.refpunkt = Refpunkt(self)

        self.config = Config(self)
        self.gpc = GPS_calc()


    def addConnections(self):
        self.minmax.set_callback(self.gpc)
        self.refpunkt.set_callback(self.gpc)

        # die Serielle Schnittstelle muss zuerst geöffnet werden,
        # damit die Referenzen darauf weitergereicht werden
        self.ser_config()
        self.receiver = Receiver(callback = self.readdata, gui = self, rx = self.rx)


    def ser_config(self):
        """ wird kurz nach dem Start der Applikation automatisch aufgerufen und
            versucht, eine freie Serielle Schnittstelle zu öffnen.
        """
        portname, self.rx = open_serport(self)
        if self.rx is not None:
            App.Titel_1 = "{} @ {}".format(App.Titel_1, portname)
            self.title(App.Titel_1)

        else:
            messagebox.showerror("GPS", "Es wurde kein COM-Port geöffnet !")


    def gps_config(self):
        """ Die GPS-Maus konfigurieren """
        self.config.config_window(self.rx)


    def logfile(self):
        """ Ein Logfile öffnen bzw. schließen """
        self.filename, self.handle = file_open(self.handle)

        if self.handle is not None:
            self.log_file.set(1)
            App.Titel_2 = "  [Logfile - {}]".format(self.filename)
            t = "{}{}".format(App.Titel_1, App.Titel_2)
            self.title(t)

        else:
            self.log_file.set(0)
            App.Titel_2 = ""
            self.title(App.Titel_1)
            messagebox.showinfo("GPS", "Die Logdatei wurde geschlossen !")

        # den Receiver informieren, ob Daten geloggt werden sollen
        self.receiver.set_connection(self.handle)


    def logpause(self):
        """ Das Loggen unterbrechen """
        if self.log_activ.get():
            self.title("{}{}".format(App.Titel_1, App.Titel_2))
            self.receiver.setpause(0)

        else:
            self.title("{} - [Pause]".format(App.Titel_1))
            self.receiver.setpause(1)


    def ende(self):
        self.bell()
        if messagebox.askyesno("GBPS", "Programm beenden"):
            self.destroy()


    def __shutdown(self, event):
        """ wird einmalig aufgerufen, wenn Tk mit dem 'X' geschlossen wird
            aber auch dann, wenn über self.ende self.destroy() aufgerufen wird.
            Also läuft jedes Beenden über __shutdown ! """
        print("shutdown")
        self.receiver.quit = True
        if self.handle:
            self.handle.close()
            self.handle = None
            print("file closed")


    def __resetref(self, *args):
        self.bell()
        self.gpc.set_refpunkt()


    def __resetminmax(self, *args):
        self.bell()
        self.gpc.clr_minmax()


    def readdata(self, id, data):
        """ wird aufgerufen, wenn ein Datensatz empfangen worden ist,
            zuerst wird geprüft, welches Format vorliegt,
            dann werden die Daten ins zugehörige Fenster gesendet
            und dort angezeigt """

        if id == "RMC":
            self.counter.add((0, 1))

            self.rmc.updateRMC(self.gpc.read_RMC(data))

            # wenn der Abstand zum Referenzpunkt angezeigt werden soll
            if self.refpunkt.show.get():
                self.refpunkt.update(self.gpc._calc_ref_dist())

            # wenn die Standortabweichung angezeigt werden soll
            if self.minmax.show.get():
                self.minmax.update(self.gpc._calc_dist())

        elif id == "VTG":
            self.counter.add((3, 1))
            if self.rmc.showVTG.get():
                self.rmc.updateVTG(self.gpc.read_VTG(data))

        elif id == "GSA":
            self.counter.add((1, 1))
            self.gsa.update(self.gpc.read_GSA(data))

        elif id == "GGA":
            self.counter.add((2, 1))
            self.gga.update(self.gpc.read_GGA(data))

        elif id == "GSV":
            temp =self.gpc.read_GSV(data)
            if temp is not None:
                self.counter.add((4,temp[1]))
                self.gsv.update(temp[0])

        elif id == "GLL":
            self.counter.add((5,1))
            self.gll.update(self.gpc.read_GLL(data))


class Receiver(Thread):
    """ Ein Hintergrundthread zum Empfang der GPS-Daten
        via Serieller Schnittstelle """
    def __init__(self, callback, gui, rx = None):
        Thread.__init__(self)
        self.callback = callback
        self.rx = rx
        self.gui = gui
        self.handle = None
        self.pause = 0
        self.quit = False

        self.start()

    def set_connection(self, handle):
        self.handle = handle


    def stop(self):
        """ Beendet den Thread nach dem nächsten Sleep() """
        self.quit = True


    def setpause(self, mode):
        """ Der Thread reicht empfangene Daten nicht weiter """
        self.pause = mode


    def run(self):
        # warten, bis eine Serielle Schnittstelle geöffnet ist
        while self.rx is None:
            time.sleep(.5)

        self.rx.flush()
        i = self.rx.inWaiting()
        while i > 30:
            self.rx.readline()
            i = self.rx.inWaiting()

        while True:
            if self.quit:
                print("thread RX quit")
                exit()

            # wenn die Auswertung unterbleiben soll
            if self.pause:
                time.sleep(1)
                x = self.rx.inWaiting()
                while x > 30:
                    self.rx.readline()
                    x = self.rx.inWaiting()
                continue

            if self.rx.inWaiting() > 30:
                try:
                    zeile = self.rx.readline().decode('ascii')
                except:
                    print("decoding error")
                    continue

                cnt = len(zeile)

                # Die drei relevanten Zeichen = data set identifier
                gpid = zeile[3:6]

                # Nur die Datensätze weiter bearbeiten, die auch angezeigt werden
                if ((gpid == "RMC" and self.gui.rmc.showRMC.get()) or
                        (gpid == "GSA" and self.gui.gsa.show.get()) or
                        (gpid == "GGA" and self.gui.gga.show.get()) or
                        (gpid == "VTG" and self.gui.rmc.showVTG.get()) or
                        (gpid == "GSV" and self.gui.gsv.show.get()) or
                        (gpid == "GLL" and self.gui.gll.show.get())):

                    #z = zeile[1:-2]        #  '$' + '\r\n' abschneiden
                    z = zeile[:-2]          #  ''\r\n' abschneiden

                    # ggf. den Datensatz in eine Datei schreiben
                    if self.handle is not None:
                        # passiert während des Dialogs beim Schließen der Datei
                        if not self.handle.closed:
                            self.handle.write(z)
                            self.handle.write('\r')

                    # Den String umwandeln in eine Liste, Trennzeichen ist das Komma
                    t = z.split(',')

                    # Die Prüfsumme (beginnend mit '*') entfernen
                    # sie befindet sich im letzten Element - sofern vorhanden
                    tmp = t[len(t) -1]

                    if tmp.find('*') is not -1:
                        t[len(t) -1] = tmp[:tmp.find('*')]

                    if self.callback is not None:
                        self.callback( gpid, t)


            else:   #  not self.rx.get_Bytes_Waiting() > 30
                time.sleep(0.1)

App()