#!/usr/bin/python
# -*- coding: UTF-8 -*-

from tkinter import *

# --- GPRMC --------------------------------------------------------------------
#
# ------------------------------------------------------------------------------
class RMC(LabelFrame):

    def __init__(self, master):
        """Legt die erforderlichen Variablen für die RMC-Anzeige an"""

        LabelFrame.__init__(self, text = "GPRMC / GPVTG")
        self.grid(column = 0, row = 0,  padx = 5, pady = 5, sticky = "nsew")

        self.showRMC = BooleanVar()
        self.showRMC.set(True)
        self.showVTG = BooleanVar()
        self.showVTG.set(False)

        self._vars = []
        for i in range(6):
            self._vars.append( StringVar() )

        items = ("UTC-Zeit:", "Datum:", "Geogr. Länge:", "Geogr. Breite:", "[VTG] Speed:",
                 "[VTG] Kurs:")
        row = 0
        for item in items:
            Label(self, text = item, anchor = E, width = 20).grid(
                    column = 0, row = row, padx = 10, sticky = "w")
            Label(self, textvariable = self._vars[row], anchor = W, width = 15).grid(
                    column = 1, row = row, padx = 10, sticky = "w")
            row += 1

        show1 = Checkbutton(self, text="GPRMC anzeigen", variable= self.showRMC,
                            anchor = W, width = 15, onvalue=True)
        show1.grid(column = 1, row = 6,  padx = 10, sticky = "w")
        show1.bind("<ButtonRelease>", self.__togRMC)

        show2 = Checkbutton(self, text="GPVTG anzeigen", variable=self.showVTG,
                            anchor = W, width = 15,  onvalue=True)
        show2.grid(column = 1, row = 7, padx = 10,  sticky = "e")
        show2.bind("<1>", self.__togVTG)


    def __togRMC(self, event):
        """Wird aufgerufen, wenn die Checkbox betätigt wird,
           löscht die Anzeige"""
        if self.showRMC.get():
            for var in self._vars:
                var.set("")

    def __togVTG(self, event):
        """Wird aufgerufen, wenn die Checkbox betätigt wird,
           löscht dabei die Anzeige"""
        if self.showVTG.get():
            self._vars[4].set("")
            self._vars[5].set("")


    def updateRMC(self, d):
        """Aktualisiert alle Werte im RMC-Bereich """
        for i in range(len(self._vars)):
            self._vars[i].set(d[i])


    def updateVTG(self, d):
        """ aktualisiert nur die beiden VTG-Werte """
        if self.showVTG.get():
            self._vars[4].set(d[0])
            self._vars[5].set(d[1])


# --- GPVTG - Window ------------------------------------------------------------
#
# -------------------------------------------------------------------------------
# die beiden VTG-Werte sind ins RMC - Window eingebaut
# wenn RMC nicht aktiv ist, werden die VTG-Werte angezeigt.


# --- GPGSA - Window ---------------------------------------------------------
#
# ----------------------------------------------------------------------------
class GSA(LabelFrame):

    def __init__(self, master):
        LabelFrame.__init__(self, master, text = "GPGSA")
        self.grid(column = 0, row = 1,  padx = 5, pady = 5, sticky = "nsew")

        self.show  = BooleanVar()
        self.show.set(True)

        self._vars = []
        for i in range(5):
            self._vars.append( StringVar() )

        items = ("Satellites (1 -  6):", "Satellites (7 - 12):", "Position DOP:",
                 "Horizontal DOP:", "Vertikal DOP:")

        row = 0
        for item in items:
            Label(self, text = item, anchor = E, width = 20).grid(
                    column = 0, row = row, padx = 10, sticky = "w")
            Label(self, textvariable = self._vars[row], anchor = W, width = 15).grid(
                    column = 1, row = row, padx = 10, sticky = "w")
            row += 1

        show = Checkbutton(self, text="GPGSA anzeigen", variable=self.show, anchor = W, width = 15, onvalue=True)
        show.grid(column = 1, row = 5,  padx = 10, sticky=E)
        show.bind("<1>", self.__tog)


    def __tog(self, event):
        """Wird aufgerufen, wenn die Checkbox betätigt wird
           und löscht dabei die Anzeige"""
        # Die Anzeige löschen
        if self.show.get():
            for var in self._vars:
                var.set("")


    def update(self, d):
        """Schreibt neue Werte """
        for i in range(len(self._vars)):
            self._vars[i].set(d[i])


# --- GPGGA - Window ---------------------------------------------------------------------------------------
#
# ----------------------------------------------------------------------------------------------------------

class GGA(LabelFrame):
    def __init__(self, master):
        LabelFrame.__init__(self, text = "GPGGA")
        self.grid(column = 0, row = 2,  padx = 5, pady = 5, sticky = "nsew")

        self.show    = BooleanVar()
        self.show.set(True)

        self._vars = []
        for i in range(3):
            self._vars.append( StringVar())

        items = ("Satellites used:", "Höhe NN:", "Höhe geoid:")

        row = 0
        for item in items:
            Label(self, text = item, anchor = E, width = 20).grid(
                    column = 0, row = row, padx = 10, sticky = "w")
            Label(self, textvariable = self._vars[row], anchor = W, width = 15).grid(
                    column = 1, row = row, padx = 10, sticky = "w")
            row += 1

        show = Checkbutton(self, text="GPGGA anzeigen", variable=self.show, anchor = W, width = 15, onvalue=True)
        show.grid(column = 1, row = 7, padx = 10,  sticky=E)
        show.bind("<1>", self.__tog)

    def __tog(self, event):
        """Wird aufgerufen, wenn die Checkbox betätigt wird
           und löscht dabei die Anzeige"""
        # Die Anzeige löschen
        if self.show.get():
            for var in self._vars:
                var.set("")

    def update(self, d):
        """ Aktualisiert die Anzeige,
            wenn gga_show = True """
        for i in range(len(self._vars)):
            self._vars[i].set(d[i])


# --- GPGSV - Window -------------------------------------------------------------------------------------
#
# --------------------------------------------------------------------------------------------------------
class GSV(LabelFrame):

    def __init__(self, master):
        LabelFrame.__init__(self, master, text = "GPGSV")
        self.grid(column = 1, row = 0, padx = 5, pady = 5, rowspan = 2,  sticky = "nsew")

        self.show = BooleanVar()
        self.show.set(False)

        self._vars = []
        for i in range(13):
            self._vars.append( StringVar() )

        items = (   "Satellites:",
                    "Satellite 1:",
                    "Satellite 2:",
                    "Satellite 3:",
                    "Satellite 4:",
                    "Satellite 5:",
                    "Satellite 6:",
                    "Satellite 7:",
                    "Satellite 8:",
                    "Satellite 9:",
                    "Satellite 10:",
                    "Satellite 11:",
                    "Satellite 12:" )

        row = 0
        i = 0
        for item in items:
            Label(self, text = item, anchor = E, width = 15).grid(
                    column = 0, row = row, padx = 10, sticky = "w")
            Label(self, textvariable = self._vars[i], anchor = W, width = 20).grid(
                    column = 1, row = row, padx = 10, sticky = "w")
            i += 1
            row += 1
            # Eine Leerzeile in Zeile 1 erzeugen
            if row == 1:
                Label(self).grid(column = 0, row = 1)
                row = 2

        show = Checkbutton(self, text="GPGSV anzeigen", variable=self.show,
                    anchor = W, width = 20,  onvalue=True)
        show.grid(column = 1, row = 14, padx = 10,  sticky=E)
        show.bind("<1>", self.__tog)


    def __tog(self, event):
        """Wird aufgerufen, wenn die Checkbox betätigt wird
           und löscht dabei die Anzeige"""
        # Die Anzeige löschen
        if self.show.get():
            for var in self._vars:
                var.set("")


    def update(self, data):
        for i in range(len(self._vars)):
            self._vars[i].set(data[i])


# --- GPGLL - Window -----------------------------------------------------------
#
# ------------------------------------------------------------------------------
class GLL(LabelFrame):

    def __init__(self, master):
        LabelFrame.__init__(self, master, text = "GPGLL")
        self.grid(column = 1, row = 2,  padx = 5, pady = 5,sticky = "nsew")

        self.show = BooleanVar()
        self.show.set(False)

        self._vars = []
        for i in range(3):
            self._vars.append( StringVar() )

        items = ( "Geogr. Länge:","Geogr. Breite:",  "UTC-Time:")

        row = 0
        for item in items:
            Label(self, text = item, anchor = E, width = 15).grid(
                    column = 0, row = row, padx = 10, sticky = "w")
            Label(self, textvariable = self._vars[row], anchor = W, width = 20).grid(
                    column = 1, row = row, padx = 10, sticky = "w")
            row += 1

#        Label(self, anchor = E, width = 15).grid(column = 0, row = 3, sticky = W)

        show = Checkbutton(self, text="GPGLL anzeigen", variable=self.show,
                                        anchor = W,  width = 20, onvalue=True)
        show.grid(column = 1, row = 3,  padx = 10, sticky=E)
        show.bind("<1>", self.__tog)


    def __tog(self, event):
        """Wird aufgerufen, wenn die Checkbox betätigt wird
           und löscht dabei die Anzeige"""
        # Die Anzeige löschen
        if self.show.get():
            for var in self._vars:
                var.set("")


    def update(self, data):
        for i in range(len(self._vars)):
            self._vars[i].set(data[i])


# --- Status -------------------------------------------------------------------
#
# ------------------------------------------------------------------------------
class Frm_counter(LabelFrame):

    def __init__(self, master):
        LabelFrame.__init__(self, master, text = "NMEA Datensatzzähler")
        self.grid(column = 2, row = 0,  padx = 5, pady = 5, sticky = N+S+E+W)

        self.counter = [0] * 6
        self.show = BooleanVar()
        self.show.set(True)

        self._vars = []
        for i in range(6):
            self._vars.append( StringVar() )

        items = ("RMC:", "GSA:", "GGA:", "VTG", "GSV", "GLL:")

        row = 0
        for item in items:
            Label(self, text = item, anchor = E, width = 15).grid(
                    column = 0, row = row, padx = 10, sticky = "w")
            Label(self, textvariable = self._vars[row], anchor = W, width = 20).grid(
                    column = 1, row = row, padx = 10, sticky = "w")
            row += 1

        Button(self, text="Reset", command=self.__clr_counter, width = 15).grid(
                        column=0, row=6, padx = 5, pady = 5, sticky=E)

        one = Checkbutton(self, text="Counter anzeigen", variable=self.show,
            anchor = W, width = 15, onvalue=True)
        one.grid(column = 1, row = 6, padx = 10,  sticky=E)


    def __tog(self, event):
        """ Wird aufgerufen, wenn die Checkbox betätigt wird
           und löscht dabei die Anzeige """
        # Die Anzeige löschen
        if self.show.get():
            for var in self._vars:
                var.set("")


    def __clr_counter(self):
        for i in range(len(self.counter)):
            self.counter[i] = 0
        if self.show.get():
            self.update(self.counter)


    def update(self, data):
        for i in range(len(self._vars)):
            self._vars[i].set(data[i])


    def add(self, a):
        # a[0] = Counter Index
        # a[1] = Anzahl
        self.counter[a[0]] += a[1]
        if self.show.get():
            self.update(self.counter)

# --- Window min-max -----------------------------------------------------------
#
# ------------------------------------------------------------------------------

class Minmax(LabelFrame):
    def __init__(self, master):
        LabelFrame.__init__(self, master, text = "Drift / Abstand Min-Max")
        self.grid(column = 2, row = 1,  padx = 5, pady = 5, sticky = N+S+E+W)

        self.callback = None
        self.show = BooleanVar()
        self.show.set(False)

        self._vars = []
        for i in range(3):
            self._vars.append( StringVar() )

        items = ("delta Länge:", "delta Breite:", "delta absolut:")

        row = 0
        for item in items:
            Label(self, text = item, anchor = E, width = 15).grid(
                    column = 0, row = row, padx = 10, sticky = "w")
            Label(self, textvariable = self._vars[row], anchor = W, width = 20).grid(
                    column = 1, row = row, padx = 10, sticky = "w")
            row += 1

        Button(self, text="Reset Min/Max [r]", command=self.__clr_minmax, width = 15).grid(
                        column=0, row=7, columnspan = 1, padx = 5, pady = 5, sticky=E)
        self.master.bind(sequence = '<KeyPress-r>', func = self.__clr_minmax)
        show = Checkbutton(self, text="Werte anzeigen", variable=self.show,
                    anchor = W, width = 15,  onvalue=True)
        show.grid(column = 1, row = 7, padx = 10, sticky=E)
        show.bind("<1>", self.__tog)


    def set_callback(self, callback):
        self.callback = callback


    def __tog(self, event):
        """Wird aufgerufen, wenn die Checkbox betätigt wird,
           löscht dabei die Anzeige"""
        if self.show.get():
            for var in self._vars:
                var.set("")


    def __clr_minmax(self, event = None):
        """ Setzt die Minmax-Werte zurück """
        self.callback.clr_minmax()


    def update(self, data):
        for i in range(len(self._vars)):
            self._vars[i].set("{:,.1f} m".format(data[i]))


# --- Referenzpunkt ------------------------------------------------------------
#
# ------------------------------------------------------------------------------
class Refpunkt(LabelFrame):

    def __init__(self, master):
        LabelFrame.__init__(self, master, text = "Abstand zu Referenzpunkt")
        self.grid(column = 2, row = 2,  padx = 5, pady = 5,sticky = N+S+E+W)

        self.callback = None
        self.show = BooleanVar()
        self.show.set(False)

        self._vars = []
        for i in range(3):
            self._vars.append( DoubleVar() )

        items = ("Abstand Länge:", "Abstand Breite:", "Abstand absolut:")

        row = 0
        for item in items:
            Label(self, text = item, anchor = E, width = 15).grid(
                    column = 0, row = row, padx = 10, sticky = "w")
            Label(self, textvariable = self._vars[row], anchor = W, width = 20
                    ).grid(column = 1, row = row, padx = 10, sticky = "w")
            row += 1

        Button(self, text="Ref-Pkt setzen [p]", command=self.__setref, width = 15
                    ).grid(column=0, row=5, padx = 5, pady = 5, sticky=E)
        self.master.bind(sequence = '<KeyPress-p>', func = self.__setref)
        show = Checkbutton(self, text="Abstand anzeigen", variable=self.show,
                    anchor = W, width = 15, onvalue=True)
        show.grid(column = 1, row = 5, padx = 10, sticky=E)
        show.bind("<1>", self._ref_tog)


    def set_callback(self, callback):
        self.callback = callback


    def _ref_tog(self, event):
        """ Wird aufgerufen, wenn die Checkbox betätigt wird,
            setzt den Referenzpunkt auf die aktuelle Position
            und löscht die Anzeige """
        # falls noch kein Referenzpunkt gesetzt ist, wird die aktuelle
        # Position gewählt.
        self.callback.check_refpunkt()
        if self.show.get():
            for var in self._vars:
                var.set("")


    def __setref(self, event = None):
        """ setzt einen neuen Referenzpunkt """
        self.callback.set_refpunkt()


    def update(self, data):
        for i in range(len(self._vars)):
            self._vars[i].set("{:,.1f} m".format(data[i]))

