#!/usr/bin/python3
# -*- coding: UTF-8 -*-
#       20.06.2014
#
# --- [ serdialog.py ] ---------------------------------------------------------
#
# Dialog zur Auswahl einer Seriellen Schnittstelle
#
# Öffnet ein Auswahlfenster, wenn mehr als 1 Schnittstelle gefunden worden sind
# und der defaultport in der Liste nicht enthalten ist.

from tkinter import *
from tkinter import ttk
import time

# ------------------------------------------------------------------------------
class SerDialog(Toplevel):
    """ Dialog zur Auswahl einer Seriellen Schnittstelle """
    def __init__(self, master, portlist):
        Toplevel.__init__(self, master)
        self.master = master
        self.geometry('+300+300')
        self.portlist = portlist

        self.title("Serielle Schnittstelle öffnen")

        self.bind_class(Toplevel, "<Destroy>", self.top_quit)
        self.resizable(width = False, height = False) # Größe nicht veränderbar

        self.frm = ttk.LabelFrame(self, text = "Auswahl Serielle Schnittstelle")
        self.frm.grid(column = 0, row = 0, padx = 5, pady = 5, sticky = "NSEW")

        self.combo = ttk.Combobox(self.frm, values=self.portlist,
                                state="readonly", height=2, width = 6)
        self.combo.set(self.portlist[0])
        self.combo.grid(column = 0, row = 1,padx = 5, pady = 5, sticky = "nswe")
        self.combo.bind("<<ComboboxSelected>>", self._change_selection)

        self.lbl = ttk.Label(self.frm, text = "COM-Port auswählen:")
        self.lbl.grid(column = 0, row = 0, columnspan = 2, padx = 5, pady = 5, sticky = "NSEW")

        self.btn1 = ttk.Button(self.frm, text = "Übernehmen",  width = 14, command = self.übernehmen)
        self.btn1.grid(column = 2, row = 1, padx = 5, pady = 5, sticky = "NSEW")

        self.btn2 = ttk.Button(self.frm, text = "Beenden", width = 14, command = self.top_quit)
        self.btn2.grid(column = 3, row = 1, padx = 5, pady = 5, sticky = "NSEW")


    def start(self):
        # Vorgabe für die Auswahl ist das erste Element der Liste
        # dies wird zurückgegeben, wenn keine andere Auswahl getroffen wird
        self.selection = self.portlist[0]

        self.focus_set()                # den Focus auf das Fenster setzen
        self.transient(self.master)     # das Fenster im Vordergrund vor "root" halten
        self.grab_set()                 # events gehen nur an diesen Dialog
        self.master.wait_window(self)   # warten bis das Fenster geschlossen wird

        return self.selection


    def _change_selection(self, event = None):
        # Holt die Bezeichnung des in der Liste gewählten Elementes
       j = event.widget.current()
       self.selection = self.portlist[j]
       # self.selection = event.widget.current()


    def übernehmen(self):
        print("übernehmen")
        self.destroy()


    def top_quit(self, event = None):
        print("top_quit")
        self.selection = None
        self.destroy()


    def topc_quit(self, event = None):
        print("self_quit")
        self.selection = None
        self.destroy()

def main():
    top = Tk()
    s = SerDialog(top, ("COM1", "COM2", "COM3", "COM4"))
    print("Ausgewählt wurde:", s.start())
    top.destroy()
    top.mainloop()

if __name__ == '__main__':
    main()
