#!/usr/bin/python3
# -*- coding: UTF-8 -*-
#       20.06.2014

# --- [ serport.py ] -----------------------------------------------------------
#
# Öffnet die Serielle Schnittstelle:
#
# - wenn genau eine freie Schnittstelle gefunden wird, wird diese geöffnet
# - werden mehrere gefunden, dann wird der defaultport geöffnet,
#   sofern er in der Liste der freien Schnittstellen enthalten ist.
#   Der Defaultport kann in dieser Datei definiert werden oder
#   in einer Datei mit dem Namen "comport.cfg"
#   Aus dieser Datei wird nur die erste Zeile eingelesen,
#   in der muss stehen: COMx
#   wobei x die Nummer der Seriellen Schnittstelle darstellt
# - Wird mehr als 1 offener Port gefunden und ist kein defaultport definiert,
#   dann wird ein Auswahlfenster mit allen offenen Ports angeboten

#   Rückgabe: Name des Comports, Referenz auf das Objekt
#       oder: None, None


import serial
import sys
from os.path import isfile
from serdialog import SerDialog

BAUDRATE = 4800

# ------------------------------------------------------------------------------
def open_serport(master, start = 1, end = 13, timeout = 0.1):
    """
    Prüft, ob sich die Ports COM1: bis COM13: öffnen lassen
    gibt eine Liste der öffenbaren Ports zurück.
    Falls genau 1 Port gefunden wurde, wird dieser geöffnet.

    Werden mehrere gefunden, dann kann eine Vorgabe unter
    "default_comport" festgelegt werden.
    Oder es wird eine Datei "comport.cfg" gefunden, in deren erster
    Zeile der Port angegeben ist.

    Oder es wird der Suchbereich offener Ports durch die
    Parameter 'start' und 'end' eingegrenzt:
        open_serport(start = 9, end = 10) findet nur den Port 9

    Rückgabe: Name der Schnittstlle, Referenz auf die serielle Schnittstelle
    """
    ports = []
    comport = None
    default_comport = ""

# Die Ports können als "COM1" .. "COM10" oder
# über eine Ziffer      0     ..  9 angesprochen werden

    opsys = sys.platform
    if opsys == "win32":
        for i in range(start, end):
            try:
                p  = "COM" + str(i)
                rx = serial.Serial(p)
                rx.close()
                ports.append(p)

            except IOError:
                pass

    elif op.sys == "linux2":
        # Vielleicht arbeiten wir auf einem RaspberryPi ??
        try:
            p = "/dev/ttyAMA0"
            rx = serial.Serial(p)
            rx.close()
            ports.append(p)

        except IOError:
            pass

    else:
        print("OS =", opsys )
        print("quit")
        exit()

    print("OS: {} - Freie Ports: {}".format(opsys, ports))

    if len(ports) == 0:         # Kein freier Port gefunden
        print("Kein freier Port gefunden !")
        #exit()
        return None, None

    elif len(ports) == 1:     # genau ein freier Port gefunden
        comport = serial.Serial(ports[0], BAUDRATE, rtscts = False)
        print("geöffnet :", comport.name)

    else:                   # mehrere freie Ports gefunden
        if default_comport in ports:
            comport = serial.Serial(default_comport, BAUDRATE)
            print("geöffnet : ", default_comport)

        else:

            if isfile("comport.cfg"):
                cf = open("comport.cfg")
                default_comport = (cf.readline()).rstrip()
                cf.close()

                if default_comport in ports:
                    comport = serial.Serial(default_comport, BAUDRATE)
                    print("geöffnet : ", default_comport)
                else:
                    print("Default-Port {}: konnte nicht geöffnet werden".format(
                            default_comport))
                    return None, None

            else:

                dialog = SerDialog(master, ports)
                ser = dialog.start()
                if ser is None:
                    return None, None

                comport = serial.Serial(ser, BAUDRATE)

            print("geöffnet aus Dialog: {}". format(comport.name))
    return comport.name, comport





# --- [ eof ] ------------------------------------------------------------------
